%{
  #include "regexp_lexer.h"
  #define yyterminate() return RegExpParser::token::END
  #include <iostream>

  bool DEBUG_LEXER = false;
%}

%option c++
%option yyclass="AlgorithMan::RegularExpressions::RegExpLexer"
%option full
%option noyywrap
%option nounput
%option batch
%option prefix="RegExp"
%option yylineno




LETTER [a-zA-Z]

%% // --------------------------------------------------------------------------

"("|"\\left("                            { if(DEBUG_LEXER) std::cerr << "LEXER: OPEN" << std::endl;
                                           return RegExpParser::token::OPEN; }
")"|"\\right)"                           { if(DEBUG_LEXER) std::cerr << "LEXER: CLOSE" << std::endl;
                                           return RegExpParser::token::CLOSE; }
[\n\t" "]*                               { }

"+"                                      { if(DEBUG_LEXER) std::cerr << "LEXER: PLUS" << std::endl;
                                           return RegExpParser::token::PLUS;}
"-"|"\\backslash"                        { if(DEBUG_LEXER) std::cerr << "LEXER: MINUS" << std::endl;
                                           return RegExpParser::token::MINUS;}
"*"                                      { if(DEBUG_LEXER) std::cerr << "LEXER: MULT" << std::endl;
                                           return RegExpParser::token::MULT;}
"\^*"                                    { if(DEBUG_LEXER) std::cerr << "LEXER: KLEENE" << std::endl;
                                           return RegExpParser::token::KLEENE;}
"\^+"                                    { if(DEBUG_LEXER) std::cerr << "LEXER: KLEENEPLUS" << std::endl;
                                           return RegExpParser::token::KLEENEPLUS;}
"/"                                      { if(DEBUG_LEXER) std::cerr << "LEXER: QUOT" << std::endl;
                                           return RegExpParser::token::QUOT;}
"\\cap"                                  { if(DEBUG_LEXER) std::cerr << "LEXER: INTERSECTION" << std::endl;
                                           return RegExpParser::token::INTERSECTION;}

"="                                      { if(DEBUG_LEXER) std::cerr << "LEXER: EQ" << std::endl;
                                           return RegExpParser::token::EQ;}
"\\neq"                                  { if(DEBUG_LEXER) std::cerr << "LEXER: NEQ" << std::endl;
                                           return RegExpParser::token::NEQ;}
"\\subset"                               { if(DEBUG_LEXER) std::cerr << "LEXER: SUBSET" << std::endl;
                                           return RegExpParser::token::SUBSET;}
"\\subseteq"                             { if(DEBUG_LEXER) std::cerr << "LEXER: SUBSETEQ" << std::endl;
                                           return RegExpParser::token::SUBSETEQ;}
"\\supset"                               { if(DEBUG_LEXER) std::cerr << "LEXER: SUPERSET" << std::endl;
                                           return RegExpParser::token::SUPERSET;}
"\\supseteq"                             { if(DEBUG_LEXER) std::cerr << "LEXER: SUPERSETEQ" << std::endl;
                                           return RegExpParser::token::SUPERSETEQ;}

{LETTER}+                                {
                                           if(DEBUG_LEXER) std::cerr << "LEXER: STRING" << std::endl;
                                           yylval->sval = new std::string(yytext);
                                           return RegExpParser::token::STRING;
                                         }

.|\n                                     {
                                           std::cerr << "\n\nLEXICAL ERROR (Line "
                                                << yylineno
                                                << "): Unexpected character \""
                                                << yytext[0] << "\"" << std::endl;
                                         }


%% //---------------------------------------------------------------------------


RegExpLexer::RegExpLexer(std::istream& is) : RegExpFlexLexer(&is)
{
}

int RegExpLexer::yylex(RegExpParser::semantic_type* lval)
{
    yylval = lval;
    return yylex();
}
