

%code requires {

    // Many thanks to
    // http://www.thebinaryidiot.com/archives/2010/03/06/flex-bison-cpp-example/

    #include "regular_expression.h"

    namespace AlgorithMan { namespace RegularExpressions { class RegExpLexer; } }
    using namespace AlgorithMan::RegularExpressions;
}

%skeleton "lalr1.cc"
%defines
%define api.namespace {AlgorithMan::RegularExpressions}
%define parser_class_name {RegExpParser}
%parse-param { AlgorithMan::RegularExpressions::RegExpLexer &scanner }
%parse-param { AlgorithMan::RegularExpressions::RegularExpression** result }
%lex-param { AlgorithMan::RegularExpressions::RegExpLexer &scanner }
%union
{
    std::string* sval;
    RegularExpression* xval;
};

%code {
    static int yylex(RegExpParser::semantic_type * yylval, AlgorithMan::RegularExpressions::RegExpLexer &scanner);
    bool DEBUG_PARSER = false;
}



%start S
%token OPEN CLOSE PLUS MINUS MULT KLEENE KLEENEPLUS QUOT INTERSECTION
       EQ NEQ SUBSET SUBSETEQ SUPERSET SUPERSETEQ STRING
%token END 0 "end of file"
%left SUBSET SUBSETEQ SUPERSET SUPERSETEQ EQ NEQ
%left PLUS MINUS
%left MULT QUOT INTERSECTION
%left KLEENE KLEENEPLUS
%nonassoc OPEN CLOSE STRING



%% // Produktionen
S:     regex END                     { if(DEBUG_PARSER) std::cerr << "PARSER: S -> regex" << std::endl;
                                       *result = $<xval>1; }
       ;
regex:   OPEN regex CLOSE            { if(DEBUG_PARSER) std::cerr << "PARSER: regex -> OPEN regex CLOSE" << std::endl;
                                       $<xval>$ = $<xval>2; }
       | regex PLUS regex            { if(DEBUG_PARSER) std::cerr << "PARSER: regex -> regex PLUS regex" << std::endl;
                                       $<xval>$ = new Sum($<xval>1, $<xval>3); }
//     | regex MINUS regex           { if(DEBUG_PARSER) std::cerr << "PARSER: regex -> regex MINUS regex" << std::endl;
//                                     $<xval>$ = new Difference($<xval>1, $<xval>3); }
       | regex MULT regex            { if(DEBUG_PARSER) std::cerr << "PARSER: regex -> regex MULT regex" << std::endl;
                                       $<xval>$ = new Product($<xval>1, $<xval>3); }
       | regex regex                 { if(DEBUG_PARSER) std::cerr << "PARSER: regex -> regex regex" << std::endl;
                                       $<xval>$ = new Product($<xval>1, $<xval>2); } %prec MULT
//     | regex QUOT regex            { if(DEBUG_PARSER) std::cerr << "PARSER: regex -> regex QUOT regex" << std::endl;
//                                     $<xval>$ = new Quotient($<xval>1, $<xval>3); }
       | regex KLEENE                { if(DEBUG_PARSER) std::cerr << "PARSER: regex -> regex KLEENE" << std::endl;
                                       $<xval>$ = new KleeneStar($<xval>1); }
//     | regex KLEENEPLUS            { if(DEBUG_PARSER) std::cerr << "PARSER: regex -> regex KLEENEPLUS" << std::endl;
//                                     $<xval>$ = new KleenePlus($<xval>1); }
//     | regex INTERSECTION regex    { if(DEBUG_PARSER) std::cerr << "PARSER: regex -> regex INTERSECTION regex" << std::endl;
//                                     $<xval>$ = new Intersection($<xval>1,$<xval>3);}
       | STRING                      { if(DEBUG_PARSER) std::cerr << "PARSER: regex -> STRING" << std::endl;
                                       $<xval>$ = new FixedText($<sval>1); }
       | OPEN error CLOSE;
       ;

%%



#include "regexp_lexer.h"



void RegExpParser::error(const std::string &msg)
{
    std::cerr << "Error: " << msg << std::endl;
}

static int yylex(RegExpParser::semantic_type * yylval, AlgorithMan::RegularExpressions::RegExpLexer &scanner)
{
    return scanner.yylex(yylval);
}